@@ -13,13 +13,14 @@
 
 from __future__ import print_function, unicode_literals
 import sys
+from collections import OrderedDict
 import codecs
 import re
 import bisect
 from arpeggio.utils import isstr
 import types
 
-__version__ = "1.9.2"
+__version__ = "1.10.1"
 
 if sys.version < '3':
     text = unicode
@@ -88,8 +89,8 @@ class NoMatch(Exception):
         if not self.rules:
             err_message = "Not expected input"
         else:
-            what_is_expected = ["{}".format(rule_to_exp_str(r))
-                                for r in self.rules]
+            what_is_expected = OrderedDict.fromkeys(
+                ["{}".format(rule_to_exp_str(r)) for r in self.rules])
             what_str = " or ".join(what_is_expected)
             err_message = "Expected {}".format(what_str)
 
@@ -121,31 +122,31 @@ class DebugPrinter(object):
 
     Attributes:
         debug (bool): If true debugging messages will be printed.
-        _current_ident(int): Current identation level for prints.
+        _current_indent(int): Current indentation level for prints.
     """
 
     def __init__(self, **kwargs):
 
         self.debug = kwargs.pop("debug", False)
         self.file = kwargs.pop("file", sys.stdout)
-        self._current_ident = 0
+        self._current_indent = 0
 
         super(DebugPrinter, self).__init__(**kwargs)
 
-    def dprint(self, message, ident_change=0):
+    def dprint(self, message, indent_change=0):
         """
         Handle debug message. Print to the stream specified by the 'file'
         keyword argument at the current indentation level. Default stream is
         stdout.
         """
-        if ident_change < 0:
-            self._current_ident += ident_change
+        if indent_change < 0:
+            self._current_indent += indent_change
 
-        print(("%s%s" % ("   " * self._current_ident, message)),
+        print(("%s%s" % ("   " * self._current_indent, message)),
               file=self.file)
 
-        if ident_change > 0:
-            self._current_ident += ident_change
+        if indent_change > 0:
+            self._current_indent += indent_change
 
 
 # ---------------------------------------------------------
@@ -169,6 +170,9 @@ class ParsingExpression(object):
         suppress (bool): If this is set to True than no ParseTreeNode will be
             created for this ParsingExpression. Default False.
     """
+
+    suppress = False
+
     def __init__(self, *elements, **kwargs):
 
         if len(elements) == 1:
@@ -183,7 +187,8 @@ class ParsingExpression(object):
             nodes = [nodes]
         self.nodes = nodes
 
-        self.suppress = kwargs.get('suppress', False)
+        if 'suppress' in kwargs:
+            self.suppress = kwargs['suppress']
 
         # Memoization. Every node cache the parsing results for the given input
         # positions.
@@ -558,7 +563,7 @@ class UnorderedGroup(Repetition):
 
         # Prefetching
         append = results.append
-        nodes_to_try = set(self.nodes)
+        nodes_to_try = list(self.nodes)
         sep = self.sep.parse if self.sep else None
         result = None
         sep_result = None
@@ -582,7 +587,7 @@ class UnorderedGroup(Repetition):
             c_loc_pos = parser.position
             match = True
             all_optionals_fail = True
-            for e in set(nodes_to_try):
+            for e in list(nodes_to_try):
                 try:
                     result = e.parse(parser)
                     if result:
@@ -611,7 +616,7 @@ class UnorderedGroup(Repetition):
             parser.eolterm = old_eolterm
 
         if not match:
-            # Unsucessful match of the whole PE - full backtracking
+            # Unsuccessful match of the whole PE - full backtracking
             parser.position = c_pos
             parser._nm_raise(self, c_pos, parser)
 
@@ -714,8 +719,8 @@ class Match(ParsingExpression):
     """
     Base class for all classes that will try to match something from the input.
     """
-    def __init__(self, rule_name, root=False):
-        super(Match, self).__init__(rule_name=rule_name, root=root)
+    def __init__(self, rule_name, root=False, **kwargs):
+        super(Match, self).__init__(rule_name=rule_name, root=root, **kwargs)
 
     @property
     def name(self):
@@ -806,8 +811,9 @@ class RegExMatch(Match):
 
     '''
     def __init__(self, to_match, rule_name='', root=False, ignore_case=None,
-                 multiline=None, str_repr=None, re_flags=re.MULTILINE):
-        super(RegExMatch, self).__init__(rule_name, root)
+                 multiline=None, str_repr=None, re_flags=re.MULTILINE,
+                 **kwargs):
+        super(RegExMatch, self).__init__(rule_name, root, **kwargs)
         self.to_match_regex = to_match
         self.ignore_case = ignore_case
         self.multiline = multiline
@@ -860,8 +866,9 @@ class StrMatch(Match):
         ignore_case(bool): If case insensitive match is needed.
             Default is None to support propagation from global parser setting.
     """
-    def __init__(self, to_match, rule_name='', root=False, ignore_case=None):
-        super(StrMatch, self).__init__(rule_name, root)
+    def __init__(self, to_match, rule_name='', root=False, ignore_case=None,
+                 **kwargs):
+        super(StrMatch, self).__init__(rule_name, root, **kwargs)
         self.to_match = to_match
         self.ignore_case = ignore_case
 
@@ -905,6 +912,7 @@ class StrMatch(Match):
         return hash(self.to_match)
 
 
+
 # HACK: Kwd class is a bit hackish. Need to find a better way to
 #       introduce different classes of string tokens.
 class Kwd(StrMatch):
@@ -1020,6 +1028,10 @@ class ParseTreeNode(object):
             # If default actions are enabled
             return visitor.visit__default__(self, children)
 
+    def tree_str(self, indent=0):
+        return '{}{} [{}-{}]'.format('  ' * indent, self.rule.name,
+                                     self.position, self.position_end)
+
 
 class Terminal(ParseTreeNode):
     """
@@ -1069,6 +1081,10 @@ class Terminal(ParseTreeNode):
     def __repr__(self):
         return self.desc
 
+    def tree_str(self, indent=0):
+        return '{}: {}'.format(super(Terminal, self).tree_str(indent),
+                               self.value)
+
     def __eq__(self, other):
         return text(self) == text(other)
 
@@ -1128,6 +1144,11 @@ class NonTerminal(ParseTreeNode, list):
     def __repr__(self):
         return "[ %s ]" % ", ".join([repr(x) for x in self])
 
+    def tree_str(self, indent=0):
+        return '{}\n{}'.format(super(NonTerminal, self).tree_str(indent),
+                               '\n'.join([c.tree_str(indent + 1)
+                                          for c in self]))
+
     def __getattr__(self, rule_name):
         """
         Find a child (non)terminal by the rule name.
@@ -1167,6 +1188,10 @@ class NonTerminal(ParseTreeNode, list):
                     nodes.append(n)
                     rule = n.rule
 
+        if rule is None:
+            # If rule is not found resort to default behavior
+            return self.__getattribute__(rule_name)
+
         result = NonTerminal(rule=rule, nodes=nodes, _filtered=True)
         self._expr_cache[rule_name] = result
         return result
@@ -1714,7 +1739,8 @@ class CrossRef(object):
 
 class ParserPython(Parser):
 
-    def __init__(self, language_def, comment_def=None, *args, **kwargs):
+    def __init__(self, language_def, comment_def=None, syntax_classes=None,
+                 *args, **kwargs):
         """
         Constructs parser from python statements and expressions.
 
@@ -1723,9 +1749,14 @@ class ParserPython(Parser):
                 the root rule of the grammar.
             comment_def (python function): A python function that defines
                 the root rule of the comments grammar.
+            syntax_classes (dict): Overrides of special syntax parser
+                expression classes (StrMatch, Sequence, OrderedChoice).
+
         """
         super(ParserPython, self).__init__(*args, **kwargs)
 
+        self.syntax_classes = syntax_classes if syntax_classes else {}
+
         # PEG Abstract Syntax Graph
         self.parser_model = self._from_python(language_def)
 
@@ -1758,6 +1789,10 @@ class ParserPython(Parser):
         __rule_cache = {"EndOfFile": EndOfFile()}
         __for_resolving = []  # Expressions that needs crossref resolvnih
         self.__cross_refs = 0
+        _StrMatch = self.syntax_classes.get('StrMatch', StrMatch)
+        _OrderedChoice = self.syntax_classes.get('OrderedChoice',
+                                                 OrderedChoice)
+        _Sequence = self.syntax_classes.get('Sequence', Sequence)
 
         def inner_from_python(expression):
             retval = None
@@ -1798,9 +1833,10 @@ class ParserPython(Parser):
                     self.dprint("New rule: {} -> {}"
                                 .format(rule_name, retval.__class__.__name__))
 
-            elif type(expression) is text or isinstance(expression, StrMatch):
+            elif type(expression) is text or isinstance(expression, _StrMatch):
                 if type(expression) is text:
-                    retval = StrMatch(expression, ignore_case=self.ignore_case)
+                    retval = _StrMatch(expression,
+                                       ignore_case=self.ignore_case)
                 else:
                     retval = expression
                     if expression.ignore_case is None:
@@ -1835,7 +1871,7 @@ class ParserPython(Parser):
                 if any((isinstance(x, CrossRef) for x in retval.nodes)):
                     __for_resolving.append(retval)
 
-            elif isinstance(expression, Sequence) or \
+            elif isinstance(expression, _Sequence) or \
                     isinstance(expression, Repetition) or \
                     isinstance(expression, SyntaxPredicate) or \
                     isinstance(expression, Decorator):
@@ -1846,9 +1882,9 @@ class ParserPython(Parser):
 
             elif type(expression) in [list, tuple]:
                 if type(expression) is list:
-                    retval = OrderedChoice(expression)
+                    retval = _OrderedChoice(expression)
                 else:
-                    retval = Sequence(expression)
+                    retval = _Sequence(expression)
 
                 retval.nodes = [inner_from_python(e) for e in expression]
                 if any((isinstance(x, CrossRef) for x in retval.nodes)):
